from django.conf import settings

from .choices import (CURRENCY_CHOICES as BASE_CURRENCY_CHOICES,
                      CURRENCY_LABELS as BASE_CURRENCY_LABELS,
                      CURRENCY_BY_COUNTRIES as BASE_CURRENCY_BY_COUNTRIES)


CURRENCY_COOKIE_NAME = getattr(settings, 'METASETTINGS_CURRENCY_COOKIE_NAME', 'django_currency')

DEFAULT_CURRENCY = getattr(settings, 'METASETTINGS_DEFAULT_CURRENCY', 'EUR')

CURRENCY_CHOICES = getattr(settings, 'METASETTINGS_CURRENCY_CHOICES', BASE_CURRENCY_CHOICES)

CURRENCY_LABELS = getattr(settings, 'METASETTINGS_CURRENCY_LABELS', BASE_CURRENCY_LABELS)

CURRENCY_BY_COUNTRIES = getattr(settings, 'METASETTINGS_CURRENCY_BY_COUNTRIES', BASE_CURRENCY_BY_COUNTRIES)



import sys
import requests

from datetime import date, datetime
from dateutil.relativedelta import relativedelta

from django.core.management.base import BaseCommand, CommandError

from optparse import make_option

from metasettings.models import CurrencyRate
from metasettings.settings import CURRENCY_CHOICES

try:
    import simplejson as json
except ImportError:
    import json  # NOQA


class Command(BaseCommand):
    can_import_settings = True

    option_list = BaseCommand.option_list + (
        make_option('--app_id',
                    dest='app_id',
                    default=None,
                    help='The openexchangerates APP ID'),

        make_option('--date_start',
                    dest='date_start',
                    default=None,
                    help='The date start to import currency rates'),

        make_option('--date_end',
                    dest='date_end',
                    default=None,
                    help='The date end to import currency rates'),
    )

    def handle(self, *args, **options):
        self.app_id = options.get('app_id', None)

        if not self.app_id:
            raise CommandError('The openexchangerates APP ID is required')

        start = options.get('date_start')
        end = options.get('date_end')

        if start is None and end is None:
            return self.rate_request()

        if start:
            start = datetime.strptime(options.get('date_start'), "%Y-%m-%d").date()
        else:
            start = date.today()

        if end:
            end = datetime.strptime(options.get('date_end'), "%Y-%m-%d").date()
        else:
            end = date.today()

        date_start = date(start.year, start.month, 1)
        date_end = date(end.year, end.month, 1)

        if date_start and date_end:
            current = date_start

            while date_end >= current:
                self.rate_request(current)

                current = current + relativedelta(months=1)

    def rate_request(self, current_date=None):
        url = 'http://openexchangerates.org'

        if current_date:
            url += '/api/historical/%s.json' % current_date.strftime("%Y-%m-%d")
        else:
            url += '/latest.json'

        response = requests.get(url, params={
            'app_id': self.app_id
        })

        if response.status_code == 200:
            result = json.loads(response.content)

            currency_choices = dict(CURRENCY_CHOICES)

            for currency, rate in result['rates'].iteritems():
                if not currency in currency_choices:
                    continue

                try:
                    filters = {
                        'currency': currency
                    }

                    if current_date:
                        filters.update({
                            'year': current_date.year,
                            'month': current_date.month
                        })

                    existing_rate = CurrencyRate.objects.get(**filters)

                    existing_rate.rate = str(existing_rate.rate)
                    rate = str("%.2f" % (rate))

                    if existing_rate.rate == rate:
                        continue
                    else:
                        existing_rate.rate = rate
                        existing_rate.save()

                        sys.stdout.write('Syncing currency %s with %s\n' % (currency, rate))

                except CurrencyRate.DoesNotExist:
                    currency_rate = CurrencyRate()
                    currency_rate.rate = str(rate)

                    for k, v in filters.items():
                        setattr(currency_rate, k, v)

                    currency_rate.save()

                    sys.stdout.write('Create currency %s with %s\n' % (currency, rate))



import logging
import math

from collections import defaultdict

from django.utils import translation
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django.db import models

from . import settings


def get_currency_from_request(request):
    currency_code = request.COOKIES.get(settings.CURRENCY_COOKIE_NAME, None)

    if not currency_code:
        currency_code = get_currency_from_ip_address(request.META['REMOTE_ADDR'])

    return currency_code or settings.DEFAULT_CURRENCY


def get_currency_from_ip_address(ip_address):
    currency_code = None

    try:
        from .compat import GeoIP
    except ImportError, e:
        logging.info(e)
    else:
        g = GeoIP()
        country = g.country(ip_address)

        currency_by_countries = dict(settings.CURRENCY_BY_COUNTRIES)

        if 'country_code' in country and country['country_code'] in currency_by_countries:
            currency_code = currency_by_countries.get(country['country_code'], None)

    return currency_code


def get_language_from_request(request):
    return translation.get_language_from_request(request)


def convert_amount(from_currency, to_currency, amount, ceil=False,
                   year=None, month=None):
    if from_currency == to_currency:
        return amount

    currency_rates = CurrencyRate.objects.get_currency_rates(year=year, month=month)

    result = (amount / currency_rates[from_currency].rate) * currency_rates[to_currency].rate

    if ceil:
        result = int(math.ceil(result))

    return result


class CurrencyRateManager(models.Manager):
    @cached_property
    def rates(self):
        rates = defaultdict(lambda: defaultdict(lambda: defaultdict(dict)))

        for currency_rate in self.all():
            if currency_rate.year and currency_rate.month:
                rates[currency_rate.year][currency_rate.month][currency_rate.currency] = currency_rate
            else:
                rates[currency_rate.currency] = currency_rate

        return rates

    @cached_property
    def default_rates(self):
        return dict((currency_rate.currency, currency_rate)
                    for currency_rate in self.filter(month__isnull=True, year__isnull=True))

    def get_currency_rates(self, year=None, month=None):
        if year and month:
            if year in self.rates and month in self.rates[year]:
                return self.rates[year][month]

        return self.default_rates


class CurrencyRate(models.Model):
    currency = models.CharField(max_length=3, choices=settings.CURRENCY_LABELS, verbose_name=_('Currency'))
    rate = models.DecimalField(verbose_name=_(u'Rate'), max_digits=7, decimal_places=2)

    month = models.PositiveIntegerField(null=True, blank=True)
    year = models.PositiveIntegerField(null=True, blank=True)

    date_last_sync = models.DateTimeField(auto_now_add=True, auto_now=True)

    objects = CurrencyRateManager()



from mock import patch

from datetime import date
from dateutil.relativedelta import relativedelta

from django.test import TestCase
from django.test.client import Client
from django.template import Context, Template
from django.core.management import call_command
from django.conf import settings
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from metasettings.models import (CurrencyRate, convert_amount,
                                 CurrencyRateManager,
                                 get_currency_from_ip_address)
from metasettings.settings import CURRENCY_CHOICES


class MetasettingsTests(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_sync_rates(self):
        call_command('sync_rates', app_id=settings.OPENEXCHANGERATES_APP_ID)

        self.assertEqual(CurrencyRate.objects.filter(year__isnull=True, month__isnull=True).count(),
                         len(dict(CURRENCY_CHOICES).keys()))

    def test_sync_rates_with_date_start(self):
        date_start = date.today()

        call_command('sync_rates',
                     app_id=settings.OPENEXCHANGERATES_APP_ID,
                     date_start=date_start.strftime('%Y-%m-%d'))

        self.assertEqual(CurrencyRate.objects.filter(year=date_start.year, month=date_start.month).count(),
                         len(dict(CURRENCY_CHOICES).keys()))

    def test_sync_rates_with_date_start_and_date_end(self):
        months_count = 5

        dates = []

        date_end = date.today()

        date_start = date_end - relativedelta(months=months_count)

        for i in range(months_count):
            dates.append(date_start + relativedelta(months=i))

        call_command('sync_rates',
                     app_id=settings.OPENEXCHANGERATES_APP_ID,
                     date_start=date_start.strftime('%Y-%m-%d'),
                     date_end=date_end.strftime('%Y-%m-%d'))

        total = len(dict(CURRENCY_CHOICES).keys()) * months_count

        count = 0

        for current_date in dates:
            count += CurrencyRate.objects.filter(year=current_date.year,
                                                 month=current_date.month).count()

        self.assertEqual(count, total)

    def test_convert_amount(self):
        call_command('sync_rates', app_id=settings.OPENEXCHANGERATES_APP_ID)

        with patch.object(CurrencyRateManager, 'get_currency_rates') as get_currency_rates:
            get_currency_rates.return_value = {
                'EUR': CurrencyRate(rate=0.730862),
                'USD': CurrencyRate(rate=1)
            }

            amount = convert_amount('EUR', 'USD', 15, ceil=True)

            self.assertEqual(amount, 21)

            amount = convert_amount('EUR', 'USD', 15)

            self.assertEqual('%.2f' % amount, '20.52')

            amount = convert_amount('EUR', 'USD', 15, ceil=True)

            self.assertEqual(amount, 21)

            amount = convert_amount('EUR', 'USD', 15)

            self.assertEqual('%.2f' % amount, '20.52')

        amount = convert_amount('EUR', 'USD', 15, ceil=True, year=2011, month=10)

        self.assertEqual(amount, 21)

    def test_get_currency_from_ip_address(self):
        self.assertEqual(get_currency_from_ip_address('78.192.244.8'), 'EUR')  # France
        self.assertEqual(get_currency_from_ip_address('69.197.132.80'), 'USD')  # USA
        self.assertEqual(get_currency_from_ip_address('80.193.214.232'), 'GBP')  # United Kingdom
        self.assertEqual(get_currency_from_ip_address('218.75.205.72'), 'USD')  # China
        self.assertEqual(get_currency_from_ip_address('203.152.216.75'), 'JPY')  # Japan
        self.assertEqual(get_currency_from_ip_address('187.32.127.161'), 'BRL')  # Brasil

    def test_convert_amount_templatetags(self):
        with patch.object(CurrencyRateManager, 'get_currency_rates') as get_currency_rates:
            get_currency_rates.return_value = {
                'EUR': CurrencyRate(rate=0.730862),
                'USD': CurrencyRate(rate=1)
            }

            t = Template("{% load metasettings_tags %}{% convert_amount 'EUR' 'USD' 15 1 %}")
            result = t.render(Context())

            self.assertEqual(result, u'21')

            t = Template("{% load metasettings_tags %}{% convert_amount 'EUR' 'USD' 15 ceil=1 %}")
            result = t.render(Context())

            self.assertEqual(result, u'21')

            t = Template("{% load metasettings_tags %}{% convert_amount from_currency='EUR' to_currency='USD' amount=15 %}")
            result = t.render(Context())

            self.assertEqual(result, u'20.5237103585')

            t = Template("{% load metasettings_tags %}{% convert_amount 'EUR' 'USD' 15 ceil=1 as amount %}{{ amount }}")
            result = t.render(Context())

            self.assertEqual(result, u'21')

            t = Template("{% load metasettings_tags %}{% get_currency_from_request request as currency %}{{ currency }}")
            result = t.render(Context({
                'request': self.factory.get('/')
            }))

            self.assertEqual(result, u'EUR')

            t = Template("{% load metasettings_tags %}{% get_language_from_request request as lang %}{{ lang }}")
            result = t.render(Context({
                'request': self.factory.get('/')
            }))

            self.assertEqual(result, settings.LANGUAGE_CODE)

    def test_dashboard_view(self):
        client = Client()
        response = client.post(reverse('metasettings_dashboard'))

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'metasettings/dashboard.html')

    def test_dashboard_complete(self):
        client = Client()
        response = client.post(reverse('metasettings_dashboard'), data={
            'submit': 1,
            'language_code': 'en',
            'currency_code': 'EUR',
            'redirect_url': reverse('root')
        })

        self.assertEqual(response.status_code, 302)
